/*
############################################################
"callvote" and "vote" commands
Track vote.
example:
callvote settrack 5
vote yes
############################################################
*/
var vote_id = 0;
var vote_cmdline = null;
var votes = [];

function count_votes(final) {
	var i, status, uid, aye, nay, total;

	total = 0;
	aye = 0;
	nay = 0;

	for (i = 0; i < mxserver.max_slots; i++) {
		status = mxserver.get_status(i);
		uid = mxserver.get_uid(i);
		if ((status == "Player" || status == "Spectator") && uid != 0) {
			total++;
			if (uid in votes) {
				if (votes[uid])
					aye++;
				else
					nay++;
			}
		}
	}

	mxserver.broadcast("Vote tally is " + aye.toFixed(0) + " yes, " +
	 nay.toFixed(0) + " no, " + (total - aye - nay).toFixed(0) + " abstain");

	if (aye + aye > total) {
		mxserver.broadcast("Vote succeeded!");
		mxserver.schedule_command("at +1 " + vote_cmdline);
		vote_cmdline = null;
	}

	if (nay + nay > total || final) {
		mxserver.broadcast("Vote failed!");
		vote_cmdline = null;
	}
}

function vote(slot, cmdline) {
	var uid;

	m = cmdline.match(/^\s*vote\s+(yes|no|aye|nay)\s*$/);

	if (m == null) {
		mxserver.send(slot, "Usage: vote yes|no");
		return 1;
	}

	if (vote_cmdline == null) {
		mxserver.send(slot, "No vote in progress");
		return 1;
	}

	uid = mxserver.get_uid(slot);

	if (uid != 0) {
		if (m[1] == "yes" || m[1] == "aye")
			votes[uid] = true;
		else
			votes[uid] = false;
		mxserver.broadcast(mxserver.get_slot_info(slot).name + " voted " + m[1]);
	}

	count_votes(false);

	return 1;
}

function callvote(slot, cmdline) {
	var m, tracknum, track, msg, uid;

	m = cmdline.match(/^\s*callvote\s+settrack\s+([0-9]+)\s*$/);

	if (m == null) {
		mxserver.send(slot, "Usage: callvote settrack <track number>");
		return 1;
	}

	if (vote_cmdline) {
		mxserver.send(slot, "Vote already in progress");
		return 1;
	}

	uid = mxserver.get_uid(slot);

	if (uid == 0) {
		mxserver.send(slot, "Key not verified");
		return 1;
	}

	tracknum = parseInt(m[1]);

	if (tracknum >= mxserver.get_number("track_count", 0)) {
		mxserver.send(slot, "Track number out of range");
		return 1;
	}

	mxserver.broadcast("Vote initiated.  To vote, type \"server,vote yes\" or \"server,vote no\"");

	vote_id++;
	vote_cmdline = "settrack " + tracknum.toFixed(0);
	votes = [];

	track = mxserver.get_string("track_list", tracknum);

	msg = mxserver.get_slot_info(slot).name + " has called a vote: " + vote_cmdline + " (" + track + ")";

	mxserver.broadcast(msg);

	mxserver.schedule_command("at +60 endvote " + vote_id.toFixed(0));

	votes[uid] = true;
	count_votes(false);

	return 1;
}

function endvote(slot, cmdline) {
	var uid;

	if (slot != mxserver.max_slots)
		return 1;

	m = cmdline.match(/^\s*endvote\s+([0-9]+)\s*$/);

	if (vote_cmdline == null || parseInt(m[1]) != vote_id)
		return 1;

	mxserver.broadcast("Voting period has ended!");
	count_votes(true);

	return 1;
}

function voting_cmds(slot, cmdline) {
	var m;

	if (cmdline.match(/^\s*callvote\b/) != null)
		return callvote(slot, cmdline);
	else if (cmdline.match(/^\s*vote\b/) != null)
		return vote(slot, cmdline);
	else if (cmdline.match(/^\s*endvote\b/) != null)
		return endvote(slot, cmdline);

	return voting_cmds_prev(slot, cmdline);
}

var voting_cmds_prev = mxserver.command_handler;
mxserver.command_handler = voting_cmds;
